#!/bin/sh

set -e

ksrc=`pwd`
target=../../3.2_rootfs/
target_abs=$ksrc/$target

mkdir -p ${target}lib/modules
mkdir -p ${target}boot

make uImage modules
cp arch/arm/boot/uImage ${target}boot/uImage-3

rm -rf ${target}lib/modules/*
make modules_install INSTALL_MOD_PATH=${target}
rm ${target}lib/modules/*/modules.dep
rm ${target}lib/modules/*/build
rm ${target}lib/modules/*/source
# for OE recipe..
rm -rf ${target}lib/firmware

kmoddir=`ls -d ${target}lib/modules/*/kernel/`

if test -z "$1"; then
#if true; then
#if false; then

cp_if_exists()
{
  if test -f $1; then
    cp -v $1 $2
  fi
}

# SGX
sgxbase=/home/notaz/stuff/sgx/km/
sgxbranches="1.4.14.2616 nokia 1.5.15.2766 1.6.16.3977 1.6.16.4117
 1.7.17.783851 1.7.17.867897 1.9.19.2139099 1.9.19.2188537 1.10.2359475"
for branch in $sgxbranches; do
  cd $sgxbase
  git checkout $branch
  make KERNELDIR=$ksrc clean
  make KERNELDIR=$ksrc -j2 all
  cd $ksrc
  gput=${kmoddir}drivers/gpu/pvr/$branch/
  mkdir -p $gput
  cp_if_exists $sgxbase/services4/3rdparty/bufferclass_ti/bufferclass_ti.ko $gput
  cp_if_exists $sgxbase/services4/3rdparty/dc_omap3_linux/omaplfb.ko $gput
  cp_if_exists $sgxbase/services4/3rdparty/dc_omap3430_linux/omaplfb.ko $gput
  cp_if_exists $sgxbase/services4/3rdparty/dc_omapfb3_linux/omaplfb.ko $gput
  cp_if_exists $sgxbase/pvrsrvkm.ko $gput
  # Nokia
  cp_if_exists $sgxbase/pvr/omaplfb.ko $gput
  cp_if_exists $sgxbase/pvr/pvrsrvkm.ko $gput
  cp_if_exists $sgxbase/pvr/bufferclass_ti.ko $gput

  make M=$sgxbase clean
done
mv ${kmoddir}drivers/gpu/pvr/nokia/* ${kmoddir}drivers/gpu/pvr/
rmdir ${kmoddir}drivers/gpu/pvr/nokia


# Realtek USB wifi
d=../modules/rtl8192_8188CU_linux_v3.1.2590.20110922/
make -C $d ARCH=arm KSRC=$ksrc clean
make -C $d ARCH=arm KSRC=$ksrc modules -j2
cp -v $d/8192cu.ko ${kmoddir}drivers/net/wireless/

# supposedly supports more than E, but not in same .ko?
d=../modules/rtl8192EU_linux_v4.2.2_7585.20130524/
make -C $d ARCH=arm KSRC=$ksrc clean
make -C $d ARCH=arm KSRC=$ksrc modules -j2
cp -v $d/8192eu.ko ${kmoddir}drivers/net/wireless/

d=../modules/rtl8712_8188_8191_8192SU_usb_linux_v2.6.6.0.20110401/
make -C $d ARCH=arm KSRC=$ksrc clean
make -C $d ARCH=arm KSRC=$ksrc modules -j2
cp -v $d/8712u.ko ${kmoddir}drivers/net/wireless/

d=../modules/rtl8188eu/
make -C $d ARCH=arm KSRC=$ksrc clean
make -C $d ARCH=arm KSRC=$ksrc modules -j2
make -C $d ARCH=arm KSRC=$ksrc strip
cp -v $d/8188eu.ko ${kmoddir}drivers/net/wireless/

# MT7601U (Mediatek/digitus, ED sold some..)
d=../modules/DPO_MT7601U_LinuxSTA_3.0.0.4_20130913
make -C $d ARCH=arm CROSS_COMPILE=$CROSS_COMPILE LINUX_SRC=$ksrc clean
make -C $d ARCH=arm CROSS_COMPILE=$CROSS_COMPILE LINUX_SRC=$ksrc all -j2
cp -v $d/os/linux/mt7601Usta.ko ${kmoddir}drivers/net/wireless/


# DSP stuff
mkdir ${kmoddir}drivers/dsp/
c64_tools=`pwd`/../modules/c64_tools/kmod
sed -i -e 's://#define USE_DSP_POWER_NOTIFY:#define USE_DSP_POWER_NOTIFY:' $c64_tools/kmod.h
make -C $c64_tools ARCH=arm KDIR=$ksrc clean bin
cp -v $c64_tools/c64.ko ${kmoddir}drivers/dsp/

dsplink=`pwd`/../modules/dsplink_linux_1_65_00_03/dsplink
make -C $dsplink/gpp/src ARCH=arm KERNEL_DIR=$ksrc BASE_BUILDOS=$ksrc DSPLINK=$dsplink MAKE_OPTS= \
	CC=${CROSS_COMPILE}gcc LD=${CROSS_COMPILE}ld release
cp -v $dsplink/gpp/export/BIN/Linux/OMAP3530/RELEASE/dsplinkk.ko ${kmoddir}drivers/dsp/

dsputils=`pwd`/../modules/linuxutils_2_26_01_02/packages/ti/sdo/linuxutils
make -C $dsputils/cmem/src/module ARCH=arm LINUXKERNEL_INSTALL_DIR=$ksrc MAKE_ENV= release
cp -v $dsputils/cmem/src/module/cmemk.ko ${kmoddir}drivers/dsp/
make -C $dsputils/sdma/src/module ARCH=arm LINUXKERNEL_INSTALL_DIR=$ksrc MAKE_ENV= release
cp -v $dsputils/sdma/src/module/sdmak.ko ${kmoddir}drivers/dsp/

dsp_lpm=../modules/local_power_manager_linux_1_24_02_09/packages/ti/bios/power/modules/omap3530/lpm
make -C $dsp_lpm ARCH=arm LINUXKERNEL_INSTALL_DIR=$ksrc MAKE_OPTS= DSPLINK_REPO=$dsplink/.. release
cp -v $dsp_lpm/lpm_omap3530.ko ${kmoddir}drivers/dsp/
fi


# media
media_build=`pwd`/../media_build
make -C $media_build ARCH=arm DIR=$ksrc release
make -C $media_build ARCH=arm DIR=$ksrc prepare
make -C $media_build ARCH=arm DIR=$ksrc -j3
make -C $media_build ARCH=arm DIR=$ksrc DESTDIR=${target_abs} install


rm -f ${kmoddir}../modules.dep*
cd ${target_abs}
rm -f /tmp/oe_rootfs_extra.tar.bz2
tar cjf /tmp/oe_rootfs_extra.tar.bz2 *
echo TODO scp /tmp/oe_rootfs_extra.tar.bz2 pnd13:/tmp/
