#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

struct {
	char *name;
	int offset;
}
noffs[] = {
	{ "MCBSPLP_DRR_REG",       0x00 },
	{ "MCBSPLP_DXR_REG",       0x08 },
	{ "MCBSPLP_SPCR2_REG",     0x10 },
	{ "MCBSPLP_SPCR1_REG",     0x14 },
	{ "MCBSPLP_RCR2_REG",      0x18 },
	{ "MCBSPLP_RCR1_REG",      0x1C },
	{ "MCBSPLP_XCR2_REG",      0x20 },
	{ "MCBSPLP_XCR1_REG",      0x24 },
	{ "MCBSPLP_SRGR2_REG",     0x28 },
	{ "MCBSPLP_SRGR1_REG",     0x2C },
	{ "MCBSPLP_MCR2_REG",      0x30 },
	{ "MCBSPLP_MCR1_REG",      0x34 },
	{ "MCBSPLP_RCERA_REG",     0x38 },
	{ "MCBSPLP_RCERB_REG",     0x3C },
	{ "MCBSPLP_XCERA_REG",     0x40 },
	{ "MCBSPLP_XCERB_REG",     0x44 },
	{ "MCBSPLP_PCR_REG",       0x48 },
	{ "MCBSPLP_RCERC_REG",     0x4C },
	{ "MCBSPLP_RCERD_REG",     0x50 },
	{ "MCBSPLP_XCERC_REG",     0x54 },
	{ "MCBSPLP_XCERD_REG",     0x58 },
	{ "MCBSPLP_RCERE_REG",     0x5C },
	{ "MCBSPLP_RCERF_REG",     0x60 },
	{ "MCBSPLP_XCERE_REG",     0x64 },
	{ "MCBSPLP_XCERF_REG",     0x68 },
	{ "MCBSPLP_RCERG_REG",     0x6C },
	{ "MCBSPLP_RCERH_REG",     0x70 },
	{ "MCBSPLP_XCERG_REG",     0x74 },
	{ "MCBSPLP_XCERH_REG",     0x78 },
	{ "MCBSPLP_RINTCLR_REG",   0x80 },
	{ "MCBSPLP_XINTCLR_REG",   0x84 },
	{ "MCBSPLP_ROVFLCLR_REG",  0x88 },
	{ "MCBSPLP_SYSCONFIG_REG", 0x8C },
	{ "MCBSPLP_THRSH2_REG",    0x90 },
	{ "MCBSPLP_THRSH1_REG",    0x94 },
	{ "MCBSPLP_IRQSTATUS_REG", 0xA0 },
	{ "MCBSPLP_IRQENABLE_REG", 0xA4 },
	{ "MCBSPLP_WAKEUPEN_REG",  0xA8 },
	{ "MCBSPLP_XCCR_REG",      0xAC },
	{ "MCBSPLP_RCCR_REG",      0xB0 },
	{ "MCBSPLP_XBUFFSTAT_REG", 0xB4 },
	{ "MCBSPLP_RBUFFSTAT_REG", 0xB8 },
	{ "MCBSPLP_SSELCR_REG",    0xBC },
	{ "MCBSPLP_STATUS_REG",    0xC0 },
};


void *checked_mmap(int memdev, unsigned int addr, unsigned int len)
{
	void *regs = mmap(0, len, PROT_WRITE|PROT_READ, MAP_SHARED, memdev, addr);
	if (regs == MAP_FAILED)
	{
		char err[64];
		sprintf(err, "mmap 0x%08x", addr);
		perror(err);
		exit(1);
	}

	return regs;
}

int main(int argc, char *argv[])
{
	volatile unsigned int *regs;
	int memdev, i, u;

	memdev = open("/dev/mem", O_RDWR|O_SYNC);
	if (memdev == -1)
	{
		printf("no memdev\n");
		return 1;
	}

	/* ----- clocks ----- */

	regs = checked_mmap(memdev, 0x48000000, 0x10000);

	printf("48002274 %08x CONTROL_DEVCONF0\n", regs[0x2274>>2]);
	printf("480022D8 %08x CONTROL_DEVCONF1\n", regs[0x22D8>>2]);
	printf("48004d40 %08x CM_CLKSEL1_PLL\n",   regs[0x4d40>>2]);
	printf("48005000 %08x CM_FCLKEN_PER\n",    regs[0x5000>>2]);
	printf("48005010 %08x CM_ICLKEN_PER\n",    regs[0x5010>>2]);
	printf("48005020 %08x CM_IDLEST_PER - idle status\n", regs[0x5020>>2]);
	printf("48005030 %08x CM_AUTOIDLE_PER\n",        regs[0x5030>>2]);
	printf("480022A4 %08x CONTROL_MSUSPENDMUX_5x\n", regs[0x22a4>>2]);
	printf("4800216c %08x PADCONF_MCBSP3_DX\n",   regs[0x216c>>2]);
	printf("48002170 %08x PADCONF_MCBSP3_CLKX\n", regs[0x2170>>2]);

	munmap((void *)regs, 0x10000);

	/* ----- PRCM ----- */

	regs = checked_mmap(memdev, 0x48300000, 0x10000);

	printf("483070a4 %08x PM_MPUGRPSEL_PER\n",  regs[0x70a4>>2]);
	printf("483070a8 %08x PM_IVA2GRPSEL_PER\n", regs[0x70a8>>2]);
	printf("483070b0 %08x PM_WKST_PER\n",       regs[0x70b0>>2]);

	munmap((void *)regs, 0x10000);

	/* ----- SSELCR ----- */

	regs = checked_mmap(memdev, 0x49020000, 0x10000);

	printf("4902a02c %08x ST_SSELCR_REG\n",  regs[0xa02c>>2]);

	munmap((void *)regs, 0x10000);

	/* ----- IRQs ----- */

	regs = checked_mmap(memdev, 0x48200000, 0x10000);

	// 22 ~ MCBSP3_IRQ, 89 ~ MCBSP3_IRQ_TX, 90 ~ MCBSP3_IRQ_RX
	printf("IRQs: %i, TX %i, RX %i\n", !(regs[0x0084>>2] & (1<<22)),
		!(regs[0x008c>>2] & (1<<(89-64))), !(regs[0x008c>>2] & (1<<(90-64))));

	munmap((void *)regs, 0x10000);

	/* ----- McBSP ----- */

	regs = checked_mmap(memdev, 0x49024000, 0x2000);

	printf("\n");
	for (i = 0; i < 0xC4; i+= 4)
	{
		printf("%02x %8x ", i, regs[i>>2]);
		for (u = 0; u < sizeof(noffs) / sizeof(noffs[0]); u++)
		{
			if (i == noffs[u].offset) {
				printf("%s", noffs[u].name);
				break;
			}
		}
		printf("\n");
	}

	munmap((void *)regs, 0x2000);
	close(memdev);

	return 0;
}

