#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/dma-mapping.h>

static dma_addr_t dma_addr_4m;
static void *dma_virt_4m;
static dma_addr_t dma_addr_16m;
static void *dma_virt_16m;

static int __devinit c64_tools_probe(struct platform_device *pdev)
{
	dev_info(&pdev->dev, "in probe..\n");

	dev_info(&pdev->dev, "allocating our 4M..\n");

	dma_virt_4m = dma_alloc_coherent(&pdev->dev, 4 * SZ_1M,
				&dma_addr_4m, GFP_KERNEL);
	if (dma_virt_4m == NULL) {
		dev_err(&pdev->dev, "no 4M buffer!\n");
		return -ENOMEM;
	}
	dev_info(&pdev->dev, "4M buffer ptr %p, phys %08x\n",
		dma_virt_4m, dma_addr_4m);

	if (dma_addr_4m == 0x86000000)
		dev_info(&pdev->dev, "phys location correct, yay!\n");
	else {
		dev_err(&pdev->dev, "phys location bad, boo!\n");
		goto err;
	}

	dev_info(&pdev->dev, "testing 4m buf..\n");
	memset(dma_virt_4m, 1, 4 * SZ_1M);
	dev_info(&pdev->dev, "works, yay!\n");


	dma_virt_16m = dma_alloc_coherent(NULL, 16 * SZ_1M,
				&dma_addr_16m, GFP_KERNEL);
	if (dma_virt_16m == NULL) {
		dev_err(&pdev->dev, "no 16M buffer!\n");
		goto err;
	}
	dev_info(&pdev->dev, "16M buffer ptr %p, phys %08x\n",
		dma_virt_16m, dma_addr_16m);

	dev_info(&pdev->dev, "testing 16m buf..\n");
	memset(dma_virt_16m, 1, 16 * SZ_1M);
	dev_info(&pdev->dev, "works, yay!\n");

	return 0;

err:
	dma_free_coherent(&pdev->dev, 4 * SZ_1M, dma_virt_4m, dma_addr_4m);
	return -ENOMEM;
}

static int __devexit c64_tools_remove(struct platform_device *pdev)
{
	dev_info(&pdev->dev, "freeing 4m buf..\n");
	dma_free_coherent(&pdev->dev, 4 * SZ_1M, dma_virt_4m, dma_addr_4m);

	dev_info(&pdev->dev, "freeing 16m buf..\n");
	dma_free_coherent(NULL, 16 * SZ_1M, dma_virt_16m, dma_addr_16m);

	return 0;
}

static struct platform_driver c64_tools_driver = {
	.probe		= c64_tools_probe,
	.remove		= c64_tools_remove,
	.driver		= {
		.name	= "c64_tools",
		.owner	= THIS_MODULE,
	},
};

static int init_my_module(void)
{
	printk(KERN_INFO "random test module loaded\n");

	return platform_driver_register(&c64_tools_driver);
}


static void cleanup_my_module(void)
{
	platform_driver_unregister(&c64_tools_driver);

	printk("random test module unloaded\n");
}

module_init(init_my_module);
module_exit(cleanup_my_module);

MODULE_AUTHOR("notaz");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("module test");
