#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/uaccess.h>

int  get_fpscr(void);
void set_fpscr(int val);

static int proc_read(char *page, char **start, off_t off, int count,
		int *eof, void *data)
{
	char *p = page;
	int len;

	p += sprintf(p, "%08x\n", get_fpscr());

	len = (p - page) - off;
	if (len < 0)
		len = 0;

	*eof = (len <= count) ? 1 : 0;
	*start = page + off;

	return len;
}

static int proc_write(struct file *file, const char __user *buffer,
		unsigned long count, void *data)
{
	char buff[32];
	unsigned long val;
	int ret;

	count = strncpy_from_user(buff, buffer,
			count < sizeof(buff) ? count : sizeof(buff) - 1);
	buff[count] = 0;

	ret = strict_strtoul(buff, 0, &val);
	if (ret < 0) {
		printk(KERN_ERR "error %i parsing %s\n", ret, buff);
		return ret;
	}

	set_fpscr(val);

	return count;
}

//#include <linux/clk.h>
static int init_my_module(void)
{
	struct proc_dir_entry *pret;
/*
	struct clk *xclk = clk_get(NULL, "dss1_alwon_fck");
	clk_enable(xclk);
	printk("dss1_fck: %p %i\n", xclk, clk_get_rate(xclk));
	clk_disable(xclk);
	clk_put(xclk);
	return 0;
*/
	printk(KERN_INFO "random test module loaded\n");

	pret = create_proc_entry("test",
			S_IWUSR | S_IRUGO, NULL);
	if (pret == NULL) {
		printk(KERN_ERR "can't create proc\n");
		return -1;
	}

	pret->read_proc = proc_read;
	pret->write_proc = proc_write;

	return 0;
}


static void cleanup_my_module(void)
{
	remove_proc_entry("test", NULL);
	printk("random test module unloaded\n");
}


module_init(init_my_module);
module_exit(cleanup_my_module);

MODULE_AUTHOR("notaz");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("module test");

