#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>
#include <signal.h>
#include <sys/ucontext.h>

static volatile int fault;

static void sigbus_sigaction(int num, siginfo_t *info, void *ctx)
{
	struct ucontext *context = ctx;
	unsigned int *regs = (unsigned int *)&context->uc_mcontext.arm_r0;

	//printf("\nsigbus %d %p @ %08x\n", info->si_code, info->si_addr, regs[15]);
	fault = 1;
	regs[15] += 4;

	usleep(1000);
}

int main(int argc, char *argv[])
{
	struct sigaction sigbus_action = {
		.sa_sigaction = sigbus_sigaction,
		.sa_flags = SA_SIGINFO,
	};
	unsigned int base = 0x5401d000;
	volatile unsigned int *regs;
	unsigned int val;
	int i, fd;

	fd = open("/dev/mem", O_RDONLY | O_SYNC);
	if (fd == -1)
	{
		perror("open /dev/mem");
		return 1;
	}

  	regs = mmap(0, 0x1000, PROT_READ, MAP_SHARED, fd, base);
	if (regs == MAP_FAILED) {
		close(fd);
		perror("mmap");
		return 1;
	}

	sigemptyset(&sigbus_action.sa_mask);
	sigaction(SIGBUS, &sigbus_action, NULL);

	for (i = 0; i < 1024; i++) {
		printf("%08x ", base + i * 4);

		fault = 0;

		val = regs[i];

		if (!fault)
			printf("%08x\n", val);
		else
			printf("fault\n");
		fflush(stdout);
	}

	munmap((void *)regs, 0x1000);
	close(fd);

	return 0;
}
