#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <linux/fb.h>
#include <time.h>


int devopen(const char *path)
{
	int dev;

	dev = open(path, O_RDWR|O_SYNC);
	if (dev == -1)
	{
		printf("open(\"%s\") failed with %i\n", path, errno);
		exit(1);
	}

	return dev;
}


volatile void *xmmap(int dev, int addr, int size)
{
	volatile void *ret;
	ret = mmap(0, size, PROT_READ|PROT_WRITE, MAP_SHARED, dev, addr);
	//printf("memregs are @ %p\n", memregs);
	if (ret == MAP_FAILED)
	{
		perror("mmap(memregs) failed");
		exit(1);
	}
	return ret;
}

static unsigned int get_us(void)
{
	struct timeval tv;

	gettimeofday(&tv, NULL);
	return tv.tv_sec * 1000000 + tv.tv_usec;
}

#define LOG_BUF 16

int main(int argc, char *argv[])
{
	volatile unsigned int *memregs;
	unsigned int val1 = 0, us, prev_us;
	int reads = 0;
	int memdev;
	struct {
		unsigned int val_old, val, time, rcount;
	} log[LOG_BUF];
	int i, lp = 0;


	if (argv[1] == NULL) {
		fprintf(stderr, "usage:\n%s <gpio_bank_base>\n", argv[0]);
		return 1;
	}

	memdev = devopen("/dev/mem");

	memregs = xmmap(memdev, strtoul(argv[1], NULL, 0), 0x1000);

	//memregs[0x5403c] = 0;
	//memregs[0x54090] = 0xffffffff;

	printf("SYSCONFIG: %08x\n", memregs[0x010>>2]);
	printf("OE:        %08x\n", memregs[0x034>>2]);
//	printf("LEVELDET0: %08x\n", memregs[0x040>>2]);
//	printf("LEVELDET1: %08x\n", memregs[0x044>>2]);
	printf("DEBOUNCEN: %08x\n", memregs[0x050>>2]);

//	memregs[0x54040>>2] = 0xff00;
//	memregs[0x54044>>2] = 0xff00;
	//memregs[0x54010>>2] = 0x0c; // no idle, wakeup enable
	//while (val == memregs[0x54038>>2]);

	prev_us = get_us();

	while (1)
	{
		unsigned int v1 = memregs[0x038>>2];
		reads++;

		if (v1 != val1) {
			us = get_us();
			log[lp].val_old = val1;
			log[lp].val = v1;
			log[lp].time = us - prev_us;
			log[lp].rcount = reads;
			val1 = v1;
			reads = 0;
			prev_us = us;

			lp++;
			if (lp == LOG_BUF) {
				for (i = 0; i < LOG_BUF; i++)
					printf("%08x -> %08x [%7u %d]\n",
						log[i].val_old, log[i].val,
						log[i].time, log[i].rcount);
				lp = 0;
				if (LOG_BUF > 1)
					printf("-\n");
			}
		}
		//usleep(1);
	}

	return 0;
}

