#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>
#include <linux/fb.h>
#include <time.h>


int devopen(const char *path)
{
	int dev;

	dev = open(path, O_RDWR|O_SYNC);
	if (dev == -1)
	{
		fprintf(stderr, "open(\"%s\") failed: %s\n", path, strerror(errno));
		exit(1);
	}

	return dev;
}

volatile void *xmmap(int dev, int addr, int size)
{
	volatile void *ret;
	ret = mmap(0, size, PROT_READ|PROT_WRITE, MAP_SHARED, dev, addr);
	//printf("memregs are @ %p\n", memregs);
	if (ret == MAP_FAILED)
	{
		perror("mmap(memregs) failed");
		exit(1);
	}
	return ret;
}

int main(int argc, char *argv[])
{
	volatile unsigned int *memregs;
	int memdev, sd, retval;

	if (argc != 2) {
		fprintf(stderr, "usage: %s <1|2>\n", argv[0]);
		return 1;
	}

	sd = atoi(argv[1]);
	if (sd != 1 && sd != 2) {
		fprintf(stderr, "error: please specify 1 or 2 (for SD1 and SD2)\n");
		return 1;
	}

	memdev = devopen("/dev/mem");
	memregs = xmmap(memdev, 0x49054000, 0x1000);

	retval = 0;
	if (memregs[0x0038>>2] & (1 << (30+sd-1))) {
		fprintf(stderr, "card on SD%d write protected, or pin broken\n", sd);
		retval = 1;
	}

	munmap((void *)memregs, 0x1000);
	close(memdev);

	return retval;
}


